// ============================================================
// wdjtb.cpp
//
// Creates grayscale and disabled imagelists for use with
// the MS toolbar control
//
// Fran Heeran
// ============================================================
#include <windows.h>
#include "wdjtb.h"

BOOL CALLBACK DllMain(HINSTANCE hInstDll, DWORD fdwReason,
                      LPVOID lpvReserved)
{
   switch (fdwReason) {
      case DLL_PROCESS_ATTACH:
         break;

      case DLL_PROCESS_DETACH:
         break;

      default:
         break;
   }
   return TRUE;
}

// =========================================================
// WDJLoadBitmapGrayscale()
// Loads the specified bitmap from either a file or resource
// and returns a grayscale version of the bitmap. Allows for
// a single specified color to remain unchanged allowing for
// mask colours to be preserved.
// =========================================================
HBITMAP WINAPI WDJLoadBitmapGrayscale(HINSTANCE hAppInst,
                        LPSTR pszResID, COLORREF clrTransparent,
                        UINT uFlags)
{
   LPBITMAPINFOHEADER lpBmpInfo = NULL;
   HGLOBAL hResMem  = NULL;
   HBITMAP hBMPGray = NULL;

   // Loading from a file
   if (uFlags & LR_LOADFROMFILE) {
      HANDLE hFile = CreateFile(pszResID, GENERIC_READ, 0, NULL,
                                OPEN_EXISTING, 0, NULL);
      if (INVALID_HANDLE_VALUE == hFile)
         return NULL;

      DWORD dwSize = GetFileSize(hFile, NULL);

      lpBmpInfo = (LPBITMAPINFOHEADER)malloc(dwSize);
      if (!lpBmpInfo) {
         CloseHandle(hFile);
         return NULL;
      }

      DWORD dwBytesRead;

      // Advance past the BITMAPFILEHEADER section
      SetFilePointer(hFile, sizeof(BITMAPFILEHEADER), 0,
                     FILE_BEGIN);

      // Read in the bitmap
      ReadFile(hFile, (LPVOID)lpBmpInfo, dwSize -
               sizeof(BITMAPFILEHEADER), &dwBytesRead, NULL);
      CloseHandle(hFile);
   }
   // Load the bitmap from a resource
   else {
      HRSRC hRes = ::FindResource(hAppInst, pszResID, RT_BITMAP);

      if (!hRes)
         return NULL;

      DWORD dwResSize = SizeofResource(hAppInst, hRes);

      hResMem = LoadResource(hAppInst, hRes);

      if (!hResMem)
         return NULL;

      lpBmpInfo = (LPBITMAPINFOHEADER)LockResource(hResMem);
   }

   // This function only supports 16 and 256 color bitmaps
   if (lpBmpInfo->biBitCount == 4 ||
       lpBmpInfo->biBitCount == 8) {
      // Calculate the size of the color table
      int iNumColors = (lpBmpInfo->biClrUsed) ?
                           lpBmpInfo->biClrUsed :
                           (1 << lpBmpInfo->biBitCount);

      DWORD FAR *lpColorTable = (DWORD FAR *)(lpBmpInfo + 1);

      // Convert the plette to shades of gray (skip over the
      // transparent colour if required)
      for (int i = 0;i < iNumColors;i++) {
         if (lpColorTable[i] == clrTransparent &&
             clrTransparent != CLR_NONE)
            continue;

         BYTE cRed   = GetRValue(lpColorTable[i]);
         BYTE cGreen = GetGValue(lpColorTable[i]);
         BYTE cBlue  = GetBValue(lpColorTable[i]);

         // Already a shade of gray so leave alone
         if (cRed == cGreen && cGreen == cBlue)
            continue;

         BYTE cGray =  (BYTE)(cRed / 3) + (cGreen / 2) +
                             (cBlue / 6);

         lpColorTable[i] = RGB(cGray, cGray, cGray);
      }

      // Create the gray bitmap
      HDC hScreenDC = ::GetDC(NULL);

      LPBYTE lpBits = (LPBYTE)(lpColorTable + iNumColors);

      hBMPGray = ::CreateDIBitmap(hScreenDC, lpBmpInfo,
                                  (DWORD)CBM_INIT, lpBits,
                                  (LPBITMAPINFO)lpBmpInfo,
                                  DIB_RGB_COLORS );

      ::ReleaseDC(NULL, hScreenDC);
   }

   // Free either the memory or resource
   if (HIWORD(pszResID))
      free(lpBmpInfo);
   else
      FreeResource(hResMem);

   // Return the bitmap
   return hBMPGray;
}

// ============================================================
// WDJCreateDisabledImageList()
// Takes and ImageList and the Transparent color and creates
// a disabled version as a second image list
// ============================================================
HIMAGELIST WDJCreateDisabledImageList(HIMAGELIST hImgList,
                                      COLORREF clrMask)
{
   // Create a bitmap the same size as the image list
   int iX, iY, xPos = 0;
   ImageList_GetIconSize(hImgList, &iX, &iY);

   // Get the number of images in the image list
   int iNumImages = ImageList_GetImageCount(hImgList);
   if (!iNumImages)
      return NULL;

   HDC hDCScreen, hDCTemp;
   hDCScreen = GetDC(NULL);
   hDCTemp = CreateCompatibleDC(hDCScreen);

   HBITMAP hBMPDis = CreateCompatibleBitmap(hDCScreen,
                                 iX * iNumImages, iY);
   if (hBMPDis) {
      HBITMAP hBMPOld = (HBITMAP)SelectObject(hDCTemp, hBMPDis);

      // Fill the new bitmap with the transparent color
      if (CLR_NONE != clrMask) {
         RECT rct;
         rct.top = rct.left = 0;
         rct.bottom = iY;
         rct.right = iX * iNumImages;

         SetBkColor(hDCTemp, clrMask);
         ExtTextOut(hDCTemp, 0, 0, ETO_OPAQUE | ETO_CLIPPED,
                    &rct, "", 0, NULL);
      }

      // Extract each image as an Icon in order to maintain
      // transparency and draw it into the new image list
      // as a disabled version of itself
      for (int i = 0; i < iNumImages;i++) {
         HICON hIcoImage = ImageList_GetIcon(hImgList, i,
                                             ILD_NORMAL);

         if (hIcoImage) {
            DrawState(hDCTemp, NULL, NULL, (LPARAM)hIcoImage,
                      0, xPos, 0, 0, 0,
                      DST_ICON | DSS_DISABLED);
            xPos += iX;
         }
      }
   }

   DeleteDC(hDCTemp);
   ReleaseDC(NULL, hDCScreen);

   // Create a new image list with same dimensions as the original
   HIMAGELIST hImg = ImageList_Create(iX, iY,
                           ILC_COLORDDB | ILC_MASK, iNumImages, 0);

   // Insert the bitmap into the ImageList
   ImageList_AddMasked(hImg, hBMPDis, clrMask);

   return hImg;
}

// ========================================================
// WDJLoadToolbarImageLists()
// Creates both a colour and grayscale ImageList from the
// specified resource and returns both image lists.
// The calling Application is responsible for Destroying
// these ImageLists.
// ========================================================
BOOL WINAPI WDJLoadToolbarImageLists(HINSTANCE  hInst,
                 LPSTR pszImage, int cxWidth,
                 HIMAGELIST *pHotList, HIMAGELIST *pColdList,
                 HIMAGELIST *pDisabledList, UINT uFlags,
                 COLORREF clrTransparent)
{
   if (pHotList)
      *pHotList  = NULL;
   if (pColdList)
      *pColdList = NULL;
   if (pDisabledList)
      *pDisabledList = NULL;

   // Load the Bitmap and get its dimensions
   HBITMAP  hBMPHot = (HBITMAP)LoadImage(hInst, pszImage,
                                 IMAGE_BITMAP, 0, 0,
                                 LR_DEFAULTCOLOR | uFlags);
   if (!hBMPHot)
      return FALSE;

   BITMAP bmpInfo;

   GetObject(hBMPHot, sizeof(bmpInfo), &bmpInfo);

   if (cxWidth == -1)
      cxWidth = bmpInfo.bmWidth;
   int cyHeight = bmpInfo.bmHeight;

   // Now create the grayscale version
   HBITMAP hBMPCold = WDJLoadBitmapGrayscale(hInst, pszImage,
                                             clrTransparent,
                                             uFlags);
   if (!hBMPCold) {
      DeleteObject(hBMPHot);
      return FALSE;
   }

   // Create the Hot Imagelist with and add the bitmap
   if (pHotList) {
      *pHotList = ImageList_Create(cxWidth, cyHeight,
                               ILC_COLORDDB | ILC_MASK,
                               (bmpInfo.bmWidth / cxWidth), 0);

      if (!*pHotList) {
         DeleteObject(hBMPHot);
         DeleteObject(hBMPCold);
         return FALSE;
      }

      ImageList_AddMasked(*pHotList, hBMPHot, clrTransparent);
   }

   // Create the cold image list and add the bitmap
   if (pColdList) {
      *pColdList = ImageList_Create(cxWidth, cyHeight,
                                ILC_COLORDDB | ILC_MASK,
                                (bmpInfo.bmWidth / cxWidth), 0);

      if (!*pColdList) {
         DeleteObject(hBMPHot);
         DeleteObject(hBMPCold);
         ImageList_Destroy(*pHotList);
         return FALSE;
      }
      ImageList_AddMasked(*pColdList, hBMPCold, clrTransparent);
   }

   // Create the disabled Image List
   if (pDisabledList) {
      *pDisabledList = WDJCreateDisabledImageList(*pHotList,
                                            clrTransparent);

      if (!pDisabledList)
         return FALSE;
   }

   return TRUE;
}

/* end of file */